@@ -2,10 +2,13 @@ from dataplicity.client import settings, serial
 from dataplicity.client.task import TaskManager
 from dataplicity.client.sampler import SamplerManager
 from dataplicity.client.livesettings import LiveSettingsManager
+from dataplicity.client.timeline import TimelineManager
+from dataplicity.app import comms
 from dataplicity.jsonrpc import JSONRPC
 from dataplicity import errors
 from dataplicity.constants import *
 
+
 from time import time
 import os.path
 import logging
@@ -45,10 +48,13 @@ class Client(object):
         self.tasks = TaskManager.init_from_conf(self, conf)
         self.samplers = SamplerManager.init_from_conf(self, conf)
         self.livesettings = LiveSettingsManager.init_from_conf(self, conf)
+        self.timelines = TimelineManager.init_from_conf(self, conf)
 
         self.sample_now = self.samplers.sample_now
         self.sample = self.samplers.sample
 
+        self.get_timeline = self.timelines.get_timeline
+
     def get_settings(self, name):
         self.livesettings.get(name, reload=True)
 
@@ -98,14 +104,15 @@ class Client(object):
                                "device.update_conf_map",
                                conf_map=conf_map)
 
-        batch.get_result('authenticate_result')
+            # Update timeline(s)
+            if self.timelines:
+                for timeline in self.timelines:
+                    batch.call_with_id('timeline_result_{}'.format(timeline.name),
+                                       'device.add_events',
+                                       name=timeline.name,
+                                       events=timeline.get_events())
 
-        firmware_result = batch.get_result('firmware_result')
-        if firmware_result['current']:
-            self.log.debug('firmware is current')
-        else:
-            # Install new firmware
-            pass
+        batch.get_result('authenticate_result')
 
         # Remove snapshots that were successfully synced
         # Unsuccessful snapshots remain on disk, so the next sync will re-attempt them.
@@ -117,14 +124,42 @@ class Client(object):
                 self.log.exception("Error adding samples to {} ({})".format(sampler_name, e))
             else:
                 sampler.remove_snapshot()
+
+        try:
+            changed_conf = batch.get_result("conf_result")
+        except:
+            self.log.exception('error sending settings')
+        else:
+            if changed_conf:
+                self.livesettings.update(changed_conf, self.tasks)
+                changed_conf_names = ", ".join(sorted(changed_conf.keys()))
+                self.log.debug("settings file(s) changed: {}".format(changed_conf_names))
+
+        for timeline in self.timelines:
+            try:
+                timeline_result = batch.get_result('timeline_result_{}'.format(timeline.name))
+            except:
+                self.log.exception('error sending timeline')
+            else:
+                timeline.clear_events(timeline_result)
+
         ellapsed = time() - start
         self.log.debug('sync complete {:0.2f}s'.format(ellapsed))
 
-        changed_conf = batch.get_result("conf_result")
-        if changed_conf:
-            self.livesettings.update(changed_conf, self.tasks)
-            changed_conf_names = ", ".join(sorted(changed_conf.keys()))
-            self.log.debug("settings file(s) changed: {}".format(changed_conf_names))
+        firmware_result = batch.get_result('firmware_result')
+        if firmware_result['current']:
+            self.log.debug('firmware is current')
+        else:
+            firmware_b64 = firmware_result['firmware']
+            device_class = firmware_result['device_class']
+            version = firmware_result['version']
+            self.log.debug("new firmware, version v{} for device class '{}'".format(version, device_class))
+            self.log.info("installing firmware v{}".format(version))
+            install_path = firmware.install_encoded(device_class, version, firmware_b64)
+
+            self.log('firmware installed in "{}"'.format(install_path))
+            comms = comms.Comms()
+            comms.restart()
 
 
 if __name__ == "__main__":
